/******************************************************************
Copyright (c) 1996 Novell, Inc. All Rights Reserved.

THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND
TREATIES. USE AND REDISTRIBUTION OF THIS WORK IS SUBJECT TO THE
LICENSE AGREEMENT ACCOMPANYING THE SOFTWARE DEVELOPMENT KIT (SDK)
THAT CONTAINS THIS WORK. PURSUANT TO THE SDK LICENSE AGREEMENT,
NOVELL HEREBY GRANTS TO DEVELOPER A ROYALTY-FREE, NON-EXCLUSIVE
LICENSE TO INCLUDE NOVELL'S SAMPLE CODE IN ITS PRODUCT. NOVELL
GRANTS DEVELOPER WORLDWIDE DISTRIBUTION RIGHTS TO MARKET,
DISTRIBUTE, OR SELL NOVELL'S SAMPLE CODE AS A COMPONENT OF
DEVELOPER'S PRODUCTS. NOVELL SHALL HAVE NO OBLIGATIONS TO
DEVELOPER OR DEVELOPER'S CUSTOMERS WITH RESPECT TO THIS CODE.
******************************************************************/

package com.novell.nds.dirxml.driver.jdbc.db;


import java.text.ParseException;


/**
 * A Sybase timestamp, style 109 (e&#46;g&#46; "<code>Jun 14 2001  1:15:39:583PM</code>").
 */

public
class SybaseTimestamp
    extends DBTimestamp
{
    //Jun 14 2001  1:15:39:583PM

    static private final String DEFAULT_DB_FORMAT = "109";

    static private final int LEN_SYBASE_TIMESTAMP   = 26;

    static private final int INDEX_MONTH_WHITESPACE = 3;
    static private final int INDEX_DAY_WHITESPACE   = 6;
    static private final int INDEX_YEAR_WHITESPACE  = 11;
    static private final int INDEX_HOUR_COLON       = 14;
    static private final int INDEX_MINUTES_COLON    = 17;
    static private final int INDEX_SECONDS_COLON    = 20;
    static private final int INDEX_MILLIS_COLON     = 23;

    static private final short STRING = 0;
    static private final short INT = 1;
    static private final String[][] months = {{"Jan", "01"},
                                              {"Feb", "02"},
                                              {"Mar", "03"},
                                              {"Apr", "04"},
                                              {"May", "05"},
                                              {"Jun", "06"},
                                              {"Jul", "07"},
                                              {"Aug", "08"},
                                              {"Sep", "09"},
                                              {"Oct", "10"},
                                              {"Nov", "11"},
                                              {"Dec", "12"}};

    static private
    String
    PrependZero(String number)
    {
        if(number.length() == 1)
        {
            return ("0" + number);
        }
        else
        {
            return number;
        }
    }//PrependZero(String)


    static private
    String
    GetMilitaryHour(String hour)
    {
        int hourAsInt;

        try
        {
            hourAsInt = Integer.parseInt(hour);
            if(hourAsInt > 12)
            {
                return null;
            }
            else
            {
                hourAsInt += 12;
                return String.valueOf(hourAsInt);
            }
        }//try
        catch(NumberFormatException nfe)
        {
            return null;
        }
    }//GetMilitaryHour(String, String)


    static private
    String
    GetMonthAsInt(String monthAsString)
    {
        for(int m=0; m<months.length; m++)
        {
            if(months[m][STRING].equalsIgnoreCase(monthAsString))
            {
                return months[m][INT];
            }
        }//for

        return null;
    }//GetMonthAsInt(String)


    /**
     * Instantiates a Sybase timestamp, style 109.
     *
     * @param sybTimestamp may be <code>null</code>;
     * format should be <code>mon dd yyyy hh:mm:ss:fffAM</code>
     */
    public
    SybaseTimestamp(String sybTimestamp)
    {
        this(sybTimestamp,
             DEFAULT_CLASS.getName(),
             DEFAULT_DB_FORMAT.toString(),
             DEFAULT_IN_FORMAT.toString());
    }//SybaseTimestamp()


    /**
     * Instantiates a Sybase timestamp, style 109.
     *
     * @param sybTimestamp may be <code>null</code>;
     * format should be <code>mon dd yyyy hh:mm:ss:fffAM</code>
     * @param sqlClass should be <code>java.sql.Timestamp</code>,
     * <code>java.sql.Date</code> or <code>java.sql.Time</code>;
     * the default value is <code>java.sql.Timestamp</code>
     * @param inFormat this parameter is currently ignored and
     * reserved for future use
     * @param outFormat should be "<code>1</code>" (integer) or
     * "<code>2</code>" (string); the default value is
     * "<code>1</code>" (integer); may be <code>null</code>
     */
    public
    SybaseTimestamp
    (
        String sybTimestamp,
        String sqlClass,
        String inFormat,
        String outFormat
    )
    {
        super(sybTimestamp, sqlClass, inFormat, outFormat);
    }//SybaseTimestamp()


    //assumes timestamp style 109
    //e.g. Jun 14 2001  1:15:39:583PM
    public
    void
    setDSTime(String sybTimestamp)
        throws  ParseException, IllegalArgumentException
    {
        int length;
        String value;
        int amPMIndex;
        boolean pm;
        String amPM;
        StringBuffer odbcTimestamp;

        length = sybTimestamp.length();
        if(length < LEN_SYBASE_TIMESTAMP)
        {
            throw new ParseException("Wrong length.  Expected length " + LEN_SYBASE_TIMESTAMP +
                                     ".  Actual length " + length, 0);
        }

        //mon dd yyyy  1:15:39:583PM
        //to
        //yyyy-mm-dd hh:mm:ss.fffffffff

        amPMIndex = length - 2;
        amPM = sybTimestamp.substring(amPMIndex, length);
        if(amPM.equalsIgnoreCase("PM"))
        {
            pm = true;
        }
        else if(amPM.equalsIgnoreCase("AM"))
        {
            pm = false;
        }
        else
        {
            throw new ParseException
                ("Missing AM or PM designator @ position " + amPMIndex + ".", 0);
        }

        odbcTimestamp = new StringBuffer(LEN_ODBC_TIMESTAMP);

        //append year (yyyy-)
        value = sybTimestamp.substring((INDEX_DAY_WHITESPACE+1), INDEX_YEAR_WHITESPACE);
        value = value.trim();
        odbcTimestamp.append(value + Constants.STR_HYPHEN);

        String month;

        //append month (mm-)
        value = sybTimestamp.substring(0, INDEX_MONTH_WHITESPACE);
        value = value.trim();
        month = GetMonthAsInt(value);
        if(month == null)
        {
            throw new ParseException
                ("Bad month '" + value + "'.", 0);

        }
        odbcTimestamp.append(month + Constants.STR_HYPHEN);

        //append day (dd )
        value = sybTimestamp.substring((INDEX_MONTH_WHITESPACE+1), INDEX_DAY_WHITESPACE);
        value = value.trim();
        value = PrependZero(value);
        odbcTimestamp.append(value + Constants.STR_WHITESPACE);

        //append hours (hh)
        value = sybTimestamp.substring((INDEX_YEAR_WHITESPACE+1), INDEX_HOUR_COLON);
        value = value.trim();
        if(pm)
        {
            String hour;

            hour = GetMilitaryHour(value);
            if(hour == null)
            {
                throw new ParseException
                    ("Bad hour '" + value + "'.", 0);
            }
            else
            {
                odbcTimestamp.append(hour);
            }
        }//if
        else //am
        {
            value = PrependZero(value);
            odbcTimestamp.append(value);
        }//else

        //append minutes (:mm)
        value = sybTimestamp.substring((INDEX_HOUR_COLON+1), INDEX_MINUTES_COLON);
        value = value.trim();
        value = PrependZero(value);
        odbcTimestamp.append(Constants.STR_COLON + value);

        //append seconds (:ss)
        value = sybTimestamp.substring((INDEX_MINUTES_COLON+1), INDEX_SECONDS_COLON);
        value = value.trim();
        value = PrependZero(value);
        odbcTimestamp.append(Constants.STR_COLON + value);

        setTimestamp(DBLib.NewTimestamp(odbcTimestamp.toString()));
    }//setDSTime(String)

}//class SybaseTimestamp
